@@ -34,8 +34,6 @@ from .helpers import (
     ensure_list,
     record_error,
     hash_function,
-    output_from_inputfields,
-    output_names_from_inputfields,
 )
 from .helpers_file import copyfile_input, template_update
 from .graph import DiGraph
@@ -133,18 +131,39 @@ class TaskBase:
         if not self.input_spec:
             raise Exception("No input_spec in class: %s" % self.__class__.__name__)
         klass = make_klass(self.input_spec)
-        # todo should be used to input_check in spec??
+
         self.inputs = klass(
             **{
+                # in attrs names that starts with "_" could be set when name provided w/o "_"
                 (f.name[1:] if f.name.startswith("_") else f.name): f.default
                 for f in attr.fields(klass)
             }
         )
+
         self.input_names = [
             field.name
             for field in attr.fields(klass)
             if field.name not in ["_func", "_graph_checksums"]
         ]
+
+        if inputs:
+            if isinstance(inputs, dict):
+                # selecting items that are in input_names (ignoring fields that are not in input_spec)
+                inputs = {k: v for k, v in inputs.items() if k in self.input_names}
+            # TODO: this needs to finished and tested after #305
+            elif Path(inputs).is_file():
+                inputs = json.loads(Path(inputs).read_text())
+            # TODO: this needs to finished and tested after #305
+            elif isinstance(inputs, str):
+                if self._input_sets is None or inputs not in self._input_sets:
+                    raise ValueError(f"Unknown input set {inputs!r}")
+                inputs = self._input_sets[inputs]
+
+        self.inputs = attr.evolve(self.inputs, **inputs)
+
+        # checking if metadata is set properly
+        self.inputs.check_metadata()
+        self.state_inputs = inputs
         # dictionary to save the connections with lazy fields
         self.inp_lf = {}
         self.state = None
@@ -154,18 +173,6 @@ class TaskBase:
         self._done = False
         if self._input_sets is None:
             self._input_sets = {}
-        if inputs:
-            if isinstance(inputs, dict):
-                inputs = {k: v for k, v in inputs.items() if k in self.input_names}
-            elif Path(inputs).is_file():
-                inputs = json.loads(Path(inputs).read_text())
-            elif isinstance(inputs, str):
-                if self._input_sets is None or inputs not in self._input_sets:
-                    raise ValueError(f"Unknown input set {inputs!r}")
-                inputs = self._input_sets[inputs]
-            self.inputs = attr.evolve(self.inputs, **inputs)
-            self.inputs.check_metadata()
-            self.state_inputs = inputs
 
         self.audit = Audit(
             audit_flags=audit_flags,
@@ -303,10 +310,8 @@ class TaskBase:
 
     @property
     def output_names(self):
-        """Get the names of the parameters generated by the task."""
-        output_spec_names = [f.name for f in attr.fields(make_klass(self.output_spec))]
-        from_input_spec_names = output_names_from_inputfields(self.inputs)
-        return output_spec_names + from_input_spec_names
+        """Get the names of the outputs generated by the task."""
+        return [f.name for f in attr.fields(make_klass(self.output_spec))]
 
     @property
     def can_resume(self):
@@ -394,7 +399,9 @@ class TaskBase:
             odir.mkdir(parents=False, exist_ok=True if self.can_resume else False)
             orig_inputs = attr.asdict(self.inputs)
             map_copyfiles = copyfile_input(self.inputs, self.output_dir)
-            modified_inputs = template_update(self.inputs, map_copyfiles)
+            modified_inputs = template_update(
+                self.inputs, self.output_dir, map_copyfiles=map_copyfiles
+            )
             if modified_inputs:
                 self.inputs = attr.evolve(self.inputs, **modified_inputs)
             self.audit.start_audit(odir)
@@ -403,7 +410,7 @@ class TaskBase:
             try:
                 self.audit.monitor()
                 self._run_task()
-                result.output = self._collect_outputs()
+                result.output = self._collect_outputs(output_dir=odir)
             except Exception as e:
                 record_error(self.output_dir, e)
                 result.errored = True
@@ -412,18 +419,20 @@ class TaskBase:
                 self.hooks.post_run_task(self, result)
                 self.audit.finalize_audit(result)
                 save(odir, result=result, task=self)
-                for k, v in orig_inputs.items():
-                    setattr(self.inputs, k, v)
+                # # function etc. shouldn't change anyway, so removing
+                orig_inputs = dict(
+                    (k, v) for (k, v) in orig_inputs.items() if not k.startswith("_")
+                )
+                self.inputs = attr.evolve(self.inputs, **orig_inputs)
                 os.chdir(cwd)
         self.hooks.post_run(self, result)
         return result
 
-    def _collect_outputs(self):
+    def _collect_outputs(self, output_dir):
         run_output = self.output_
-        self.output_spec = output_from_inputfields(self.output_spec, self.inputs)
         output_klass = make_klass(self.output_spec)
         output = output_klass(**{f.name: None for f in attr.fields(output_klass)})
-        other_output = output.collect_additional_outputs(self.inputs, self.output_dir)
+        other_output = output.collect_additional_outputs(self.inputs, output_dir)
         return attr.evolve(output, **run_output, **other_output)
 
     def split(self, splitter, overwrite=False, **kwargs):
@@ -985,7 +994,22 @@ class Workflow(TaskBase):
             )
 
         self._connections += new_connections
-        fields = [(name, ty.Any) for name, _ in self._connections]
+        fields = []
+        for con in self._connections:
+            wf_out_nm, lf = con
+            task_nm, task_out_nm = lf.name, lf.field
+            if task_out_nm == "all_":
+                help_string = f"all outputs from {task_nm}"
+                fields.append((wf_out_nm, dict, {"help_string": help_string}))
+            else:
+                # getting information about the output field from the task output_spec
+                # providing proper type and some help string
+                task_output_spec = getattr(self, task_nm).output_spec
+                out_fld = attr.fields_dict(make_klass(task_output_spec))[task_out_nm]
+                help_string = (
+                    f"{out_fld.metadata.get('help_string', '')} (from {task_nm})"
+                )
+                fields.append((wf_out_nm, out_fld.type, {"help_string": help_string}))
         self.output_spec = SpecInfo(name="Output", fields=fields, bases=(BaseSpec,))
         logger.info("Added %s to %s", self.output_spec, self)
 
