@@ -18,7 +18,16 @@ import inspect
 import warnings
 
 
-from .specs import Runtime, File, Directory, attr_fields, Result, LazyField
+from .specs import (
+    Runtime,
+    File,
+    Directory,
+    attr_fields,
+    Result,
+    LazyField,
+    MultiOutputObj,
+    MultiInputObj,
+)
 from .helpers_file import hash_file, hash_dir, copyfile, is_existing_file
 
 
@@ -44,10 +53,13 @@ def ensure_list(obj, tuple2list=False):
     """
     if obj is None:
         return []
-    if isinstance(obj, list):
+    # list or numpy.array (this might need some extra flag in case an array has to be converted)
+    elif isinstance(obj, list) or hasattr(obj, "__array__"):
         return obj
     elif tuple2list and isinstance(obj, tuple):
         return list(obj)
+    elif isinstance(obj, list):
+        return obj
     return [obj]
 
 
@@ -143,8 +155,14 @@ def copyfile_workflow(wf_path, result):
     """ if file in the wf results, the file will be copied to the workflow directory"""
     for field in attr_fields(result.output):
         value = getattr(result.output, field.name)
-        new_value = _copyfile_single_value(wf_path=wf_path, value=value)
-        if new_value != value:
+        # if the field is a path or it can contain a path _copyfile_single_value is run
+        # to move all files and directories to the workflow directory
+        if field.type in [File, Directory, MultiOutputObj] or type(value) in [
+            list,
+            tuple,
+            dict,
+        ]:
+            new_value = _copyfile_single_value(wf_path=wf_path, value=value)
             setattr(result.output, field.name, new_value)
     return result
 
@@ -235,13 +253,12 @@ def make_klass(spec):
         newfields = dict()
         for item in fields:
             if len(item) == 2:
+                name = item[0]
                 if isinstance(item[1], attr._make._CountingAttr):
-                    newfields[item[0]] = item[1]
-                    newfields[item[0]].validator(custom_validator)
+                    newfields[name] = item[1]
+                    newfields[name].validator(custom_validator)
                 else:
-                    newfields[item[0]] = attr.ib(
-                        type=item[1], validator=custom_validator
-                    )
+                    newfields[name] = attr.ib(type=item[1], validator=custom_validator)
             else:
                 if (
                     any([isinstance(ii, attr._make._CountingAttr) for ii in item])
@@ -273,6 +290,9 @@ def make_klass(spec):
                             metadata=mdata,
                             validator=custom_validator,
                         )
+            # if type has converter, e.g. MultiInputObj
+            if hasattr(newfields[name].type, "converter"):
+                newfields[name].converter = newfields[name].type.converter
         fields = newfields
     return attr.make_class(spec.name, fields, bases=spec.bases, kw_only=True)
 
@@ -291,7 +311,7 @@ def custom_validator(instance, attribute, value):
         or value is None
         or attribute.name.startswith("_")  # e.g. _func
         or isinstance(value, LazyField)
-        or tp_attr in [ty.Any, inspect._empty]
+        or tp_attr in [ty.Any, inspect._empty, MultiOutputObj, MultiInputObj]
     ):
         check_type = False  # no checking of the type
     elif isinstance(tp_attr, type) or tp_attr in [File, Directory]:
@@ -658,49 +678,35 @@ def hash_value(value, tp=None, metadata=None):
             return value
 
 
-def output_names_from_inputfields(inputs):
-    """
-    Collect outputs from input fields with output_file_template.
-
-    Parameters
-    ----------
-    inputs :
-        TODO
-
-    """
-    output_names = []
-    for fld in attr_fields(inputs):
-        if "output_file_template" in fld.metadata:
-            if "output_field_name" in fld.metadata:
-                field_name = fld.metadata["output_field_name"]
-            else:
-                field_name = fld.name
-            output_names.append(field_name)
-    return output_names
-
-
-def output_from_inputfields(output_spec, inputs):
+def output_from_inputfields(output_spec, input_spec):
     """
     Collect values from output from input fields.
+    If names_only is False, the output_spec is updated,
+    if names_only is True only the names are returned
 
     Parameters
     ----------
     output_spec :
         TODO
-    inputs :
+    input_spec :
         TODO
 
     """
-    for fld in attr_fields(inputs):
+    current_output_spec_names = [f.name for f in attr.fields(make_klass(output_spec))]
+    new_fields = []
+    for fld in attr.fields(make_klass(input_spec)):
         if "output_file_template" in fld.metadata:
-            value = getattr(inputs, fld.name)
             if "output_field_name" in fld.metadata:
                 field_name = fld.metadata["output_field_name"]
             else:
                 field_name = fld.name
-            output_spec.fields.append(
-                (field_name, attr.ib(type=File, metadata={"value": value}))
-            )
+            # not adding if the field already in teh output_spec
+            if field_name not in current_output_spec_names:
+                # TODO: should probably remove some of the keys
+                new_fields.append(
+                    (field_name, attr.ib(type=File, metadata=fld.metadata))
+                )
+    output_spec.fields += new_fields
     return output_spec
 
 
