@@ -2,6 +2,8 @@
 import attr
 from pathlib import Path
 import typing as ty
+import inspect
+import re
 
 from .helpers_file import template_update_single
 
@@ -18,6 +20,27 @@ class Directory:
     """An :obj:`os.pathlike` object, designating a folder."""
 
 
+class MultiInputObj:
+    """A ty.List[ty.Any] object, converter changes a single values to a list"""
+
+    @classmethod
+    def converter(cls, value):
+        from .helpers import ensure_list
+
+        return ensure_list(value)
+
+
+class MultiOutputObj:
+    """A ty.List[ty.Any] object, converter changes an 1-el list to the single value"""
+
+    @classmethod
+    def converter(cls, value):
+        if isinstance(value, list) and len(value) == 1:
+            return value[0]
+        else:
+            return value
+
+
 @attr.s(auto_attribs=True, kw_only=True)
 class SpecInfo:
     """Base data structure for metadata of specifications."""
@@ -35,6 +58,21 @@ class SpecInfo:
 class BaseSpec:
     """The base dataclass specs for all inputs and outputs."""
 
+    def __setattr__(self, name, value):
+        """changing settatr, so the converter and validator is run
+        if input is set after __init__
+        """
+        if inspect.stack()[1][3] == "__init__":  # or name.startswith("_"):
+            super().__setattr__(name, value)
+        else:
+            tp = attr.fields_dict(self.__class__)[name].type
+            # if the type has a converter, e.g., MultiInputObj
+            if hasattr(tp, "converter"):
+                value = tp.converter(value)
+            super().__setattr__(name, value)
+            # validate all fields that have set a validator
+            attr.validate(self)
+
     def collect_additional_outputs(self, inputs, output_dir):
         """Get additional outputs."""
         return {}
@@ -73,12 +111,57 @@ class BaseSpec:
         for field, value in temp_values.items():
             setattr(self, field, value)
 
+    def check_fields_input_spec(self):
+        """
+        Check fields from input spec based on the medatada.
+
+        e.g., if xor, requires are fulfilled, if value provided when mandatory.
+
+        """
+        fields = attr_fields(self)
+        names = []
+        require_to_check = {}
+        for fld in fields:
+            mdata = fld.metadata
+            # checking if the mandatory field is provided
+            if getattr(self, fld.name) is attr.NOTHING:
+                if mdata.get("mandatory"):
+                    raise AttributeError(
+                        f"{fld.name} is mandatory, but no value provided"
+                    )
+                else:
+                    continue
+            names.append(fld.name)
+
+            # checking if fields meet the xor and requires are
+            if "xor" in mdata:
+                if [el for el in mdata["xor"] if (el in names and el != fld.name)]:
+                    raise AttributeError(
+                        f"{fld.name} is mutually exclusive with {mdata['xor']}"
+                    )
+
+            if "requires" in mdata:
+                if [el for el in mdata["requires"] if el not in names]:
+                    # will check after adding all fields to names
+                    require_to_check[fld.name] = mdata["requires"]
+
+            if fld.type is File:
+                self._file_check_n_bindings(fld)
+
+        for nm, required in require_to_check.items():
+            required_notfound = [el for el in required if el not in names]
+            if required_notfound:
+                raise AttributeError(f"{nm} requires {required_notfound}")
+
+    def _file_check_n_bindings(self, field):
+        """for tasks without container, this is simple check if the file exists"""
+        file = Path(getattr(self, field.name))
+        if not file.exists():
+            raise AttributeError(f"the file from the {field.name} input does not exist")
+
     def check_metadata(self):
         """Check contained metadata."""
 
-    def check_fields_input_spec(self):
-        """Check input fields."""
-
     def template_update(self):
         """Update template."""
 
@@ -163,6 +246,56 @@ class RuntimeSpec:
     network: bool = False
 
 
+@attr.s(auto_attribs=True, kw_only=True)
+class FunctionSpec(BaseSpec):
+    """Specification for a process invoked from a shell."""
+
+    def check_metadata(self):
+        """
+        Check the metadata for fields in input_spec and fields.
+
+        Also sets the default values when available and needed.
+
+        """
+        supported_keys = {
+            "allowed_values",
+            "copyfile",
+            "help_string",
+            "mandatory",
+            # "readonly", #likely not needed
+            # "output_field_name", #likely not needed
+            # "output_file_template", #likely not needed
+            "requires",
+            "keep_extension",
+            "xor",
+            "sep",
+        }
+        # special inputs, don't have to follow rules for standard inputs
+        special_input = ["_func", "_graph_checksums"]
+
+        fields = [fld for fld in attr_fields(self) if fld.name not in special_input]
+        for fld in fields:
+            mdata = fld.metadata
+            # checking keys from metadata
+            if set(mdata.keys()) - supported_keys:
+                raise AttributeError(
+                    f"only these keys are supported {supported_keys}, but "
+                    f"{set(mdata.keys()) - supported_keys} provided"
+                )
+            # checking if the help string is provided (required field)
+            if "help_string" not in mdata:
+                raise AttributeError(f"{fld.name} doesn't have help_string field")
+            # not allowing for default if the field is mandatory
+            if not fld.default == attr.NOTHING and mdata.get("mandatory"):
+                raise AttributeError(
+                    "default value should not be set when the field is mandatory"
+                )
+            # setting default if value not provided and default is available
+            if getattr(self, fld.name) is None:
+                if not fld.default == attr.NOTHING:
+                    setattr(self, fld.name, fld.default)
+
+
 @attr.s(auto_attribs=True, kw_only=True)
 class ShellSpec(BaseSpec):
     """Specification for a process invoked from a shell."""
@@ -250,56 +383,9 @@ class ShellSpec(BaseSpec):
                 if not fld.default == attr.NOTHING:
                     setattr(self, fld.name, fld.default)
 
-    def check_fields_input_spec(self):
-        """
-        Check fields from input spec based on the medatada.
-
-        e.g., if xor, requires are fulfilled, if value provided when mandatory.
-
-        """
-        fields = attr_fields(self)
-        names = []
-        require_to_check = {}
-        for fld in fields:
-            mdata = fld.metadata
-            # checking if the mandatory field is provided
-            if getattr(self, fld.name) is attr.NOTHING:
-                if mdata.get("mandatory"):
-                    raise AttributeError(
-                        f"{fld.name} is mandatory, but no value provided"
-                    )
-                else:
-                    continue
-            names.append(fld.name)
-
-            # checking if fields meet the xor and requires are
-            if "xor" in mdata:
-                if [el for el in mdata["xor"] if el in names]:
-                    raise AttributeError(
-                        f"{fld.name} is mutually exclusive with {mdata['xor']}"
-                    )
-
-            if "requires" in mdata:
-                if [el for el in mdata["requires"] if el not in names]:
-                    # will check after adding all fields to names
-                    require_to_check[fld.name] = mdata["requires"]
-
-            if fld.type is File:
-                self._file_check(fld)
-
-        for nm, required in require_to_check.items():
-            required_notfound = [el for el in required if el not in names]
-            if required_notfound:
-                raise AttributeError(f"{nm} requires {required_notfound}")
-
-    def _file_check(self, field):
-        file = Path(getattr(self, field.name))
-        if not file.exists():
-            raise AttributeError(f"the file from the {field.name} input does not exist")
-
 
 @attr.s(auto_attribs=True, kw_only=True)
-class ShellOutSpec(BaseSpec):
+class ShellOutSpec:
     """Output specification of a generic shell process."""
 
     return_code: int
@@ -331,7 +417,7 @@ class ShellOutSpec(BaseSpec):
                             fld, inputs, output_dir
                         )
                 else:
-                    raise Exception("not implemented")
+                    raise Exception("not implemented (collect_additional_output)")
         return additional_out
 
     def _field_defaultvalue(self, fld, output_dir):
@@ -364,6 +450,9 @@ class ShellOutSpec(BaseSpec):
 
     def _field_metadata(self, fld, inputs, output_dir):
         """Collect output file if metadata specified."""
+        if self._check_requires(fld, inputs) is False:
+            return attr.NOTHING
+
         if "value" in fld.metadata:
             return output_dir / fld.metadata["value"]
         # this block is only run if "output_file_template" is provided in output_spec
@@ -371,12 +460,81 @@ class ShellOutSpec(BaseSpec):
         # than the field already should have value
         elif "output_file_template" in fld.metadata:
             inputs_templ = attr.asdict(inputs)
-            value = template_update_single(fld, inputs_templ, spec_type="output")
-            return output_dir / value
+            value = template_update_single(
+                fld, inputs_templ, output_dir=output_dir, spec_type="output"
+            )
+            return Path(value)
         elif "callable" in fld.metadata:
             return fld.metadata["callable"](fld.name, output_dir)
         else:
-            raise Exception("not implemented")
+            raise Exception("(_field_metadata) is not a current valid metadata key.")
+
+    def _check_requires(self, fld, inputs):
+        """ checking if all fields from the requires and template are set in the input
+            if requires is a list of list, checking if at least one list has all elements set
+        """
+        from .helpers import ensure_list
+
+        if "requires" in fld.metadata:
+            # if requires is a list of list it is treated as el[0] OR el[1] OR...
+            if all([isinstance(el, list) for el in fld.metadata["requires"]]):
+                field_required_OR = fld.metadata["requires"]
+            # if requires is a list of tuples/strings - I'm creating a 1-el nested list
+            elif all([isinstance(el, (str, tuple)) for el in fld.metadata["requires"]]):
+                field_required_OR = [fld.metadata["requires"]]
+            else:
+                raise Exception(
+                    f"requires field can be a list of list, or a list "
+                    f"of strings/tuples, but {fld.metadata['requires']} "
+                    f"provided for {fld.name}"
+                )
+        else:
+            field_required_OR = [[]]
+
+        for field_required in field_required_OR:
+            # if the output has output_file_template field, adding all input fields from the template to requires
+            if "output_file_template" in fld.metadata:
+                inp_fields = re.findall("{\w+}", fld.metadata["output_file_template"])
+                field_required += [
+                    el[1:-1] for el in inp_fields if el[1:-1] not in field_required
+                ]
+
+        # it's a flag, of the field from the list is not in input it will be changed to False
+        required_found = True
+        for field_required in field_required_OR:
+            required_found = True
+            # checking if the input fields from requires have set values
+            for inp in field_required:
+                if isinstance(inp, str):  # name of the input field
+                    if not hasattr(inputs, inp):
+                        raise Exception(
+                            f"{inp} is not a valid input field, can't be used in requires"
+                        )
+                    elif getattr(inputs, inp) in [attr.NOTHING, None]:
+                        required_found = False
+                        break
+                elif isinstance(inp, tuple):  # (name, allowed values)
+                    inp, allowed_val = inp[0], ensure_list(inp[1])
+                    if not hasattr(inputs, inp):
+                        raise Exception(
+                            f"{inp} is not a valid input field, can't be used in requires"
+                        )
+                    elif getattr(inputs, inp) not in allowed_val:
+                        required_found = False
+                        break
+                else:
+                    raise Exception(
+                        f"each element of the requires element should be a string or a tuple, "
+                        f"but {inp} is found in {field_required}"
+                    )
+            # if the specific list from field_required_OR has all elements set, no need to check more
+            if required_found:
+                break
+
+        if required_found:
+            return True
+        else:
+            return False
 
 
 @attr.s(auto_attribs=True, kw_only=True)
@@ -406,7 +564,7 @@ class ContainerSpec(ShellSpec):
     ] = attr.ib(default=None, metadata={"help_string": "bindings"})
     """Mount points to be bound into the container."""
 
-    def _file_check(self, field):
+    def _file_check_n_bindings(self, field):
         file = Path(getattr(self, field.name))
         if field.metadata.get("container_path"):
             # if the path is in a container the input should be treated as a str (hash as a str)
