@@ -14,12 +14,12 @@ class State:
         and input values for specific task states
         (specified by the splitter and the input).
       * It also contains information about the final groups and the final splitter
-        if combiner is available..
+        if combiner is available.
 
     Attributes
     ----------
     name : :obj:`str`
-        name of the state that is the same as name of the task
+        name of the state that is the same as a name of the task
     splitter : :obj:`str`, :obj:`tuple`, :obj:`list`
         can be a str (name of a single input),
         tuple for scalar splitter, or list for outer splitter
@@ -72,12 +72,12 @@ class State:
 
     def __init__(self, name, splitter=None, combiner=None, other_states=None):
         """
-        Initialize state.
+        Initialize a state.
 
         Parameters
         ----------
         name : :obj:`str`
-            name (should be the same as task name)
+            name (should be the same as the task's name)
         splitter : :obj:`str`, or :obj:`tuple`, or :obj:`list`
             splitter of a task
         combiner : :obj:`str`, or :obj:`list`)
@@ -119,24 +119,9 @@ class State:
         else:
             self._splitter = None
 
-    @property
-    def splitter_rpn_final(self):
-        if self.combiner:
-            _splitter_rpn_final = hlpst.remove_inp_from_splitter_rpn(
-                deepcopy(self.splitter_rpn),
-                self.right_combiner_all + self.left_combiner_all,
-            )
-            return _splitter_rpn_final
-        else:
-            return self.splitter_rpn
-
-    @property
-    def splitter_final(self):
-        """ final splitter, after removing the combined fields"""
-        return hlpst.rpn2splitter(self.splitter_rpn_final)
-
     @property
     def splitter_rpn(self):
+        """splitter in :abbr:`RPN (Reverse Polish Notation)`"""
         _splitter_rpn = hlpst.splitter2rpn(
             self.splitter, other_states=self.other_states
         )
@@ -144,6 +129,10 @@ class State:
 
     @property
     def splitter_rpn_compact(self):
+        """splitter in :abbr:`RPN (Reverse Polish Notation)`
+        with a compact representation of the prev-state part (i.e. without unwrapping
+        the part that comes from the previous states), e.g., [_NA, _NB, \*]
+        """
         if self.other_states:
             _splitter_rpn_compact = hlpst.splitter2rpn(
                 self.splitter, other_states=self.other_states, state_fields=False
@@ -153,58 +142,85 @@ class State:
             return self.splitter_rpn
 
     @property
-    def right_splitter(self):
-        """ current state splitter (i.e. the Right part)"""
-        lr_flag = self._left_right_check(self.splitter)
-        if lr_flag == "Left":
+    def splitter_final(self):
+        """the final splitter, after removing the combined fields"""
+        return hlpst.rpn2splitter(self.splitter_rpn_final)
+
+    @property
+    def splitter_rpn_final(self):
+        if self.combiner:
+            _splitter_rpn_final = hlpst.remove_inp_from_splitter_rpn(
+                deepcopy(self.splitter_rpn),
+                self.current_combiner_all + self.prev_state_combiner_all,
+            )
+            return _splitter_rpn_final
+        else:
+            return self.splitter_rpn
+
+    @property
+    def current_splitter(self):
+        """the current part of the splitter,
+        i.e. the part that is related to the current task's state only
+        (doesn't include fields propagated from the previous tasks)
+        """
+        lr_flag = self._prevst_current_check(self.splitter)
+        if lr_flag == "prev-state":
             return None
-        elif lr_flag == "Right":
+        elif lr_flag == "current":
             return self.splitter
-        elif lr_flag == "[Left, Right]":
+        elif lr_flag == "[prev-state, current]":
             return self.splitter[1]
 
     @property
-    def right_splitter_rpn(self):
-        if self.right_splitter:
-            right_splitter_rpn = hlpst.splitter2rpn(
-                self.right_splitter, other_states=self.other_states
+    def current_splitter_rpn(self):
+        """the current part of the splitter using RPN"""
+        if self.current_splitter:
+            current_splitter_rpn = hlpst.splitter2rpn(
+                self.current_splitter, other_states=self.other_states
             )
-            return right_splitter_rpn
+            return current_splitter_rpn
         else:
             return []
 
     @property
-    def left_splitter(self):
-        """ splitters from the previous stated (i.e. the Light part)"""
-        if hasattr(self, "_left_splitter"):
-            return self._left_splitter
+    def prev_state_splitter(self):
+        """ the prev-state part of the splitter,
+        i.e. the part that comes from the previous tasks' states
+        """
+        if hasattr(self, "_prev_state_splitter"):
+            return self._prev_state_splitter
         else:
             return None
 
     @property
-    def left_splitter_rpn(self):
-        if self.left_splitter:
-            left_splitter_rpn = hlpst.splitter2rpn(
-                self.left_splitter, other_states=self.other_states
+    def prev_state_splitter_rpn(self):
+        """the prev-state art of the splitter using RPN"""
+        if self.prev_state_splitter:
+            prev_state_splitter_rpn = hlpst.splitter2rpn(
+                self.prev_state_splitter, other_states=self.other_states
             )
-            return left_splitter_rpn
+            return prev_state_splitter_rpn
         else:
             return []
 
     @property
-    def left_splitter_rpn_compact(self):
-        # left rpn part, but keeping the names of the nodes, e.g. [_NA, _NB, *]
-        if self.left_splitter:
-            left_splitter_rpn_compact = hlpst.splitter2rpn(
-                self.left_splitter, other_states=self.other_states, state_fields=False
+    def prev_state_splitter_rpn_compact(self):
+        """ the prev-state part of the splitter using RPN in a compact form,
+        (without unwrapping the states from previous nodes), e.g. [_NA, _NB, \*]
+        """
+        if self.prev_state_splitter:
+            prev_state_splitter_rpn_compact = hlpst.splitter2rpn(
+                self.prev_state_splitter,
+                other_states=self.other_states,
+                state_fields=False,
             )
-            return left_splitter_rpn_compact
+            return prev_state_splitter_rpn_compact
         else:
             return []
 
     @property
     def combiner(self):
-        """Get the combiner associated to the state."""
+        """the combiner associated to the state."""
         return self._combiner
 
     @combiner.setter
@@ -217,36 +233,52 @@ class State:
             self._combiner = []
 
     @property
-    def right_combiner(self):
+    def current_combiner(self):
+        """the current part of the combiner,
+        i.e. the part that is related to the current task's state only
+        (doesn't include fields propagated from the previous tasks)
+        """
         return [comb for comb in self.combiner if self.name in comb]
 
     @property
-    def left_combiner(self):
-        if hasattr(self, "_left_combiner"):
-            return self._left_combiner
+    def current_combiner_all(self):
+        """the current part of the combiner including all the fields
+        that should be combined (i.e. not only the fields that are explicitly
+        set, but also the fields that re in the same group/axis and had to be combined
+        together, e.g., if splitter is (a, b) a and b has to be combined together)
+        """
+        if hasattr(self, "_current_combiner_all"):
+            return self._current_combiner_all
         else:
-            return list(set(self.combiner) - set(self.right_combiner))
+            return self.current_combiner
 
     @property
-    def right_combiner_all(self):
-        if hasattr(self, "_right_combiner_all"):
-            return self._right_combiner_all
+    def prev_state_combiner(self):
+        """ the prev-state part of the combiner,
+        i.e. the part that comes from the previous tasks' states
+        """
+        if hasattr(self, "_prev_state_combiner"):
+            return self._prev_state_combiner
         else:
-            return self.right_combiner
+            return list(set(self.combiner) - set(self.current_combiner))
 
     @property
-    def left_combiner_all(self):
-        if hasattr(self, "_left_combiner_all"):
-            return self._left_combiner_all
+    def prev_state_combiner_all(self):
+        """the prev-state part of the combiner including all the fields
+        that should be combined (i.e. not only the fields that are explicitly
+        set, but also the fields that re in the same group/axis and had to be combined
+        together, e.g., if splitter is (a, b) a and b has to be combined together)
+        """
+        if hasattr(self, "_prev_state_combiner_all"):
+            return list(set(self._prev_state_combiner_all))
         else:
-            return self.left_combiner
-
-    @left_combiner_all.setter
-    def left_combiner_all(self, left_combiner_all):
-        self._left_combiner_all = list(set(left_combiner_all))
+            return self.prev_state_combiner
 
     @property
     def other_states(self):
+        """ specifies the connections with previous states, uses dictionary:
+        {name of a previous state: (previous state, input field from current state)}
+        """
         return self._other_states
 
     @other_states.setter
@@ -266,7 +298,10 @@ class State:
 
     @property
     def inner_inputs(self):
-        """input fields from previous nodes"""
+        """specifies connections between fields from the current state
+        with the specific state from the previous states, uses dictionary
+        ``{input name for current state: the previous state}``
+        """
         if self.other_states:
             _inner_inputs = {}
             for name, (st, inp) in self.other_states.items():
@@ -277,7 +312,15 @@ class State:
             return {}
 
     def update_connections(self, new_other_states=None, new_combiner=None):
-        """ updating states connections and input groups"""
+        """ updating connections, can use a new other_states and combiner
+
+        Parameters
+        ----------
+        new_other_states : :obj:`dict`, optional
+            dictionary with new other_states, will be set before updating connections
+        new_combiner : :obj:`str`, or :obj:`list`, optional
+            new combiner
+        """
         if new_other_states:
             self.other_states = new_other_states
         self._connect_splitters()
@@ -286,61 +329,83 @@ class State:
 
     def _connect_splitters(self):
         """
-        Connect splitters from previous nodes.
-        Evaluates Left (the part from previous states) and Right (current state) parts.
-        If left splitter is not provided the splitter has to be completed.
-
+        Connect splitters from the previous nodes.
+        Evaluates the prev-state part of the splitter (i.e. the part from the previous states)
+        and the current part of the splitter(i.e., the current state).
+        If the prev-state splitter is not provided the splitter has to be completed.
         """
-        # TODO: should this be in the left_Splitter property?
+        # TODO: should this be in the prev-state_Splitter property?
         if self.splitter:
-            # if splitter is string, have to check if this is Left or Right part (Left is required)
+            # if splitter is string, have to check if this is prev-state or current part (prev-state is required)
             if isinstance(self.splitter, str):
-                # so this is the Left part
+                # so this is the prev-state part
                 if self.splitter.startswith("_"):
-                    self._left_splitter = self._complete_left(left=self.splitter)
-                else:  # this is Right part
-                    self._left_splitter = self._complete_left()
+                    self._prev_state_splitter = self._complete_prev_state(
+                        prev_state=self.splitter
+                    )
+                else:  # this is the current part
+                    self._prev_state_splitter = self._complete_prev_state()
             elif isinstance(self.splitter, (tuple, list)):
-                lr_flag = self._left_right_check(self.splitter)
-                if lr_flag == "Left":
-                    self._left_splitter = self._complete_left(left=self.splitter)
-                elif lr_flag == "Right":
-                    self._left_splitter = self._complete_left()
-                elif lr_flag == "[Left, Right]":
-                    self._left_splitter = self._complete_left(left=self.splitter[0])
+                lr_flag = self._prevst_current_check(self.splitter)
+                if lr_flag == "prev-state":
+                    self._prev_state_splitter = self._complete_prev_state(
+                        prev_state=self.splitter
+                    )
+                elif lr_flag == "current":
+                    self._prev_state_splitter = self._complete_prev_state()
+                elif lr_flag == "[prev-state, current]":
+                    self._prev_state_splitter = self._complete_prev_state(
+                        prev_state=self.splitter[0]
+                    )
         else:
-            # if there is no splitter, I create the Left part
-            self._left_splitter = self._complete_left()
+            # if there is no splitter, I create the prev-state part
+            self._prev_state_splitter = self._complete_prev_state()
 
-        if self.right_splitter:
+        if self.current_splitter:
             self.splitter = [
-                deepcopy(self._left_splitter),
-                deepcopy(self.right_splitter),
+                deepcopy(self._prev_state_splitter),
+                deepcopy(self.current_splitter),
             ]
         else:
-            self.splitter = deepcopy(self._left_splitter)
+            self.splitter = deepcopy(self._prev_state_splitter)
 
-    def _complete_left(self, left=None):
-        """Add all splitters from previous nodes (completing the Left part)."""
-        if left:
-            rpn_left = hlpst.splitter2rpn(
-                left, other_states=self.other_states, state_fields=False
+    def _complete_prev_state(self, prev_state=None):
+        """Add all splitters from the previous nodes (completing the prev-state part).
+
+        Parameters
+        ----------
+        prev_state :  :obj:`str`, or :obj:`list`, or :obj:`tuple`, optional
+            the prev-state part of the splitter, that has to be completed
+        """
+        if prev_state:
+            rpn_prev_state = hlpst.splitter2rpn(
+                prev_state, other_states=self.other_states, state_fields=False
             )
             for name, (st, inp) in list(self.other_states.items())[::-1]:
-                if "_{}".format(name) not in rpn_left and st.splitter_final:
-                    left = ["_{}".format(name), left]
+                if f"_{name}" not in rpn_prev_state and st.splitter_final:
+                    prev_state = [f"_{name}", prev_state]
         else:
-            left = ["_{}".format(name) for name in self.other_states]
-            if len(left) == 1:
-                left = left[0]
-        return left
+            prev_state = [f"_{name}" for name in self.other_states]
+            if len(prev_state) == 1:
+                prev_state = prev_state[0]
+        return prev_state
 
-    def _left_right_check(self, splitter_part, check_nested=True):
+    def _prevst_current_check(self, splitter_part, check_nested=True):
         """
-        Check if splitter_part is purely Left, Right
-        or [Left, Right] if the splitter_part is a list (outer splitter)
+        Check if splitter_part is purely prev-state part, the current part,
+        or mixed ([prev-state, current]) if the splitter_part is a list (outer splitter)
 
-        String is returned.
+        Parameters
+        ----------
+        splitter_part : :obj:`str`, or :obj:`list`, or :obj:`tuple`
+            Part of the splitter that is being check
+        check_nested : :obj:`bool`, optional
+            If True, the nested parts are checked.
+
+        Returns
+        -------
+        str
+           describes the type - "prev-state" or "current"
 
         If the splitter_part is mixed exception is raised.
 
@@ -353,42 +418,56 @@ class State:
             True if el.startswith("_") else False for el in inputs_in_splitter
         ]
         if all(others_in_splitter):
-            return "Left"
+            return "prev-state"
         elif (not all(others_in_splitter)) and (not any(others_in_splitter)):
-            return "Right"
+            return "current"
         elif (
             isinstance(self.splitter, list)
             and check_nested
-            and self._left_right_check(self.splitter[0], check_nested=False) == "Left"
-            and self._left_right_check(self.splitter[1], check_nested=False) == "Right"
+            and self._prevst_current_check(self.splitter[0], check_nested=False)
+            == "prev-state"
+            and self._prevst_current_check(self.splitter[1], check_nested=False)
+            == "current"
         ):
-            return "[Left, Right]"  # Left and Right parts separated in outer scalar
+            return (
+                "[prev-state, current]"
+            )  # the prev-state and the current parts separated in outer scalar
         else:
             raise hlpst.PydraStateError(
-                "Left and Right splitters are mixed - splitter invalid"
+                "prev-state and current splitters are mixed - splitter invalid"
             )
 
     def set_input_groups(self, state_fields=True):
-        """Evaluate groups, especially the final groups that address the combiner."""
-        right_splitter_rpn = hlpst.splitter2rpn(
-            self.right_splitter,
+        """Evaluates groups, especially the final groups that address the combiner.
+
+        Parameters
+        ----------
+        state_fields : :obj:`bool`
+            if False the splitter from the previous states are unwrapped
+        """
+        current_splitter_rpn = hlpst.splitter2rpn(
+            self.current_splitter,
             other_states=self.other_states,
             state_fields=state_fields,
         )
-        # merging groups from previous nodes if any input come from previous the nodes
+        # merging groups from previous nodes if any input come from previous states
         if self.inner_inputs:
             self._merge_previous_groups()
         keys_f, group_for_inputs_f, groups_stack_f, combiner_all = hlpst.splits_groups(
-            right_splitter_rpn,
-            combiner=self.right_combiner,
+            current_splitter_rpn,
+            combiner=self.current_combiner,
             inner_inputs=self.inner_inputs,
         )
-        self._right_combiner_all = combiner_all
-        if self.left_splitter and state_fields:  # if splitter has also the left part
-            self._right_keys_final = keys_f
-            self._right_group_for_inputs_final = group_for_inputs_f
-            self._right_groups_stack_final = groups_stack_f
-            if self.right_splitter:  # if Right part, adding groups from current st
+        self._current_combiner_all = combiner_all
+        if (
+            self.prev_state_splitter and state_fields
+        ):  # if splitter has also the prev-state part
+            self._current_keys_final = keys_f
+            self._current_group_for_inputs_final = group_for_inputs_f
+            self._current_groups_stack_final = groups_stack_f
+            if (
+                self.current_splitter
+            ):  # if the current part, adding groups from current st
                 self._add_current_groups()
 
         else:
@@ -402,26 +481,28 @@ class State:
         self.groups_stack_final = []
         self.group_for_inputs_final = {}
         self.keys_final = []
-        if self.left_combiner:
-            _, _, _, self.left_combiner_all = hlpst.splits_groups(
-                self.left_splitter_rpn, combiner=self.left_combiner
+        if self.prev_state_combiner:
+            _, _, _, self._prev_state_combiner_all = hlpst.splits_groups(
+                self.prev_state_splitter_rpn, combiner=self.prev_state_combiner
             )
-        for i, left_nm in enumerate(self.left_splitter_rpn_compact):
-            if left_nm in ["*", "."]:
+        for i, prev_nm in enumerate(self.prev_state_splitter_rpn_compact):
+            if prev_nm in ["*", "."]:
                 continue
             if (
-                i + 1 < len(self.left_splitter_rpn_compact)
-                and self.left_splitter_rpn_compact[i + 1] == "."
+                i + 1 < len(self.prev_state_splitter_rpn_compact)
+                and self.prev_state_splitter_rpn_compact[i + 1] == "."
             ):
                 last_gr = last_gr - 1
-            if left_nm[1:] not in self.other_states:
+            if prev_nm[1:] not in self.other_states:
                 raise hlpst.PydraStateError(
-                    f"can't ask for splitter from {left_nm[1:]}, other nodes that are connected: {self.other_states}"
+                    f"can't ask for splitter from {prev_nm[1:]}, other nodes that are connected: {self.other_states}"
                 )
-            st = self.other_states[left_nm[1:]][0]
-            # checking if left combiner contains any element from the st splitter
+            st = self.other_states[prev_nm[1:]][0]
+            # checking if prev-state combiner contains any element from the st splitter
             st_combiner = [
-                comb for comb in self.left_combiner_all if comb in st.splitter_rpn_final
+                comb
+                for comb in self.prev_state_combiner_all
+                if comb in st.splitter_rpn_final
             ]
             if not hasattr(st, "keys_final"):
                 st.set_input_groups()
@@ -443,7 +524,7 @@ class State:
                     raise hlpst.PydraStateError("previous state has to run first")
                 group_for_inputs = group_for_inputs_f_st
                 groups_stack = groups_stack_f_st
-                self.left_combiner_all += combiner_all_st
+                self._prev_state_combiner_all += combiner_all_st
             else:
                 # if no element from st.splitter is in the current combiner,
                 # using st attributes without changes
@@ -469,15 +550,15 @@ class State:
 
     def _add_current_groups(self):
         """Add additional groups from the current state."""
-        self.keys_final += self._right_keys_final
+        self.keys_final += self._current_keys_final
         nr_gr_f = max(self.group_for_inputs_final.values()) + 1
-        for inp, grs in self._right_group_for_inputs_final.items():
+        for inp, grs in self._current_group_for_inputs_final.items():
             if isinstance(grs, int):
                 grs = grs + nr_gr_f
             else:  # a list
                 grs = [gr + nr_gr_f for gr in grs]
             self.group_for_inputs_final[inp] = grs
-        for i, stack in enumerate(self._right_groups_stack_final):
+        for i, stack in enumerate(self._current_groups_stack_final):
             if i == 0:
                 stack = [gr + nr_gr_f for gr in stack]
                 self.groups_stack_final[-1] += stack
@@ -519,6 +600,12 @@ class State:
         State Values
             specific elements from inputs that can be used running interfaces
 
+        Parameters
+        ----------
+        inputs : :obj:`dict`
+            inputs of the task
+        cont_dim : :obj:`dict` or `None`
+            container's dimensions for a specific input's fields
         """
         # checking if splitter and combiner have valid forms
         self.splitter_validation()
@@ -557,12 +644,12 @@ class State:
         elements_to_remove_comb = []
         for name, (st, inp) in self.other_states.items():
             if (
-                "{}.{}".format(self.name, inp) in self.splitter_rpn
-                and "_{}".format(name) in self.splitter_rpn_compact
+                f"{self.name}.{inp}" in self.splitter_rpn
+                and f"_{name}" in self.splitter_rpn_compact
             ):
-                elements_to_remove.append("_{}".format(name))
-                if "{}.{}".format(self.name, inp) not in self.combiner:
-                    elements_to_remove_comb.append("_{}".format(name))
+                elements_to_remove.append(f"_{name}")
+                if f"{self.name}.{inp}" not in self.combiner:
+                    elements_to_remove_comb.append(f"_{name}")
 
         partial_rpn = hlpst.remove_inp_from_splitter_rpn(
             deepcopy(self.splitter_rpn_compact), elements_to_remove
@@ -591,16 +678,23 @@ class State:
         return self.states_ind
 
     def prepare_states_combined_ind(self, elements_to_remove_comb):
-        """Prepare the final list of dictionaries with indices after combiner."""
+        """Prepare the final list of dictionaries with indices after combiner.
+
+        Parameters
+        ----------
+        elements_to_remove_comb : :obj:`list`
+            elements of the splitter that should be removed due to the combining
+        """
         partial_rpn_compact = hlpst.remove_inp_from_splitter_rpn(
             deepcopy(self.splitter_rpn_compact), elements_to_remove_comb
         )
-        # combiner can have parts from the left splitter, so have to have rpn with states
+        # combiner can have parts from the prev-state splitter, so have to have rpn with states
         partial_rpn = hlpst.splitter2rpn(
             hlpst.rpn2splitter(partial_rpn_compact), other_states=self.other_states
         )
         combined_rpn = hlpst.remove_inp_from_splitter_rpn(
-            deepcopy(partial_rpn), self.right_combiner_all + self.left_combiner_all
+            deepcopy(partial_rpn),
+            self.current_combiner_all + self.prev_state_combiner_all,
         )
         if combined_rpn:
             val_r, key_r = hlpst.splits(
@@ -656,10 +750,10 @@ class State:
         if not self.other_states:
             self.inputs_ind = self.states_ind
         else:
-            # elements from the current node (the Right part)
-            if self.right_splitter_rpn:
+            # elements from the current node (the current part of the splitter)
+            if self.current_splitter_rpn:
                 values_inp, keys_inp = hlpst.splits(
-                    self.right_splitter_rpn,
+                    self.current_splitter_rpn,
                     self.inputs,
                     inner_inputs=self.inner_inputs,
                     cont_dim=self.cont_dim,
@@ -675,27 +769,25 @@ class State:
             keys_inp_prev = []
             inputs_ind_prev = []
             connected_to_inner = []
-            for ii, el in enumerate(self.left_splitter_rpn_compact):
+            for ii, el in enumerate(self.prev_state_splitter_rpn_compact):
                 if el in ["*", "."]:
                     continue
                 st, inp = self.other_states[el[1:]]
-                if (
-                    "{}.{}".format(self.name, inp) in self.splitter_rpn
-                ):  # inner splitter
+                if f"{self.name}.{inp}" in self.splitter_rpn:  # inner splitter
                     connected_to_inner += [
                         el for el in st.splitter_rpn_final if el not in [".", "*"]
                     ]
                 else:  # previous states that are not connected to inner splitter
                     st_ind = range(len(st.states_ind_final))
                     if inputs_ind_prev:
-                        # in case the Left part has scalar parts (not very well tested)
-                        if self.left_splitter_rpn_compact[ii + 1] == ".":
+                        # in case the prev-state part has scalar parts (not very well tested)
+                        if self.prev_state_splitter_rpn_compact[ii + 1] == ".":
                             inputs_ind_prev = hlpst.op["."](inputs_ind_prev, st_ind)
                         else:
                             inputs_ind_prev = hlpst.op["*"](inputs_ind_prev, st_ind)
                     else:
                         inputs_ind_prev = hlpst.op["*"](st_ind)
-                    keys_inp_prev += ["{}.{}".format(self.name, inp)]
+                    keys_inp_prev += [f"{self.name}.{inp}"]
             keys_inp = keys_inp_prev + keys_inp
 
             if inputs_ind and inputs_ind_prev:
