@@ -43,6 +43,7 @@ import cloudpickle as cp
 import inspect
 import typing as ty
 from pathlib import Path
+import warnings
 
 from .core import TaskBase, is_lazy
 from ..utils.messenger import AuditFlag
@@ -57,7 +58,13 @@ from .specs import (
     SingularitySpec,
     attr_fields,
 )
-from .helpers import ensure_list, execute, position_adjustment, argstr_formatting
+from .helpers import (
+    ensure_list,
+    execute,
+    position_adjustment,
+    argstr_formatting,
+    output_from_inputfields,
+)
 from .helpers_file import template_update, is_local_file
 
 
@@ -104,31 +111,26 @@ class FunctionTask(TaskBase):
 
         """
         if input_spec is None:
-            input_spec = SpecInfo(
-                name="Inputs",
-                fields=[
+            fields = []
+            for val in inspect.signature(func).parameters.values():
+                if val.default is not inspect.Signature.empty:
+                    val_dflt = val.default
+                else:
+                    val_dflt = attr.NOTHING
+                fields.append(
                     (
                         val.name,
                         attr.ib(
-                            default=val.default,
+                            default=val_dflt,
                             type=val.annotation,
                             metadata={
                                 "help_string": f"{val.name} parameter from {func.__name__}"
                             },
                         ),
                     )
-                    if val.default is not inspect.Signature.empty
-                    else (
-                        val.name,
-                        attr.ib(
-                            type=val.annotation, metadata={"help_string": val.name}
-                        ),
-                    )
-                    for val in inspect.signature(func).parameters.values()
-                ]
-                + [("_func", attr.ib(default=cp.dumps(func), type=str))],
-                bases=(BaseSpec,),
-            )
+                )
+            fields.append(("_func", attr.ib(default=cp.dumps(func), type=str)))
+            input_spec = SpecInfo(name="Inputs", fields=fields, bases=(BaseSpec,))
         else:
             input_spec.fields.append(
                 ("_func", attr.ib(default=cp.dumps(func), type=str))
@@ -153,6 +155,8 @@ class FunctionTask(TaskBase):
                 )
             else:
                 return_info = func.__annotations__["return"]
+                # e.g. python annotation: fun() -> ty.NamedTuple("Output", [("out", float)])
+                # or pydra decorator: @pydra.mark.annotate({"return": ty.NamedTuple(...)})
                 if hasattr(return_info, "__name__") and hasattr(
                     return_info, "__annotations__"
                 ):
@@ -161,37 +165,31 @@ class FunctionTask(TaskBase):
                         fields=list(return_info.__annotations__.items()),
                         bases=(BaseSpec,),
                     )
-                # Objects like int, float, list, tuple, and dict do not have __name__ attribute.
-                elif hasattr(return_info, "__annotations__"):
-                    output_spec = SpecInfo(
-                        name="Output",
-                        fields=list(return_info.__annotations__.items()),
-                        bases=(BaseSpec,),
-                    )
+                # e.g. python annotation: fun() -> {"out": int}
+                # or pydra decorator: @pydra.mark.annotate({"return": {"out": int}})
                 elif isinstance(return_info, dict):
                     output_spec = SpecInfo(
                         name="Output",
                         fields=list(return_info.items()),
                         bases=(BaseSpec,),
                     )
+                # e.g. python annotation: fun() -> (int, int)
+                # or pydra decorator: @pydra.mark.annotate({"return": (int, int)})
+                elif isinstance(return_info, tuple):
+                    output_spec = SpecInfo(
+                        name="Output",
+                        fields=[
+                            ("out{}".format(n + 1), t)
+                            for n, t in enumerate(return_info)
+                        ],
+                        bases=(BaseSpec,),
+                    )
+                # e.g. python annotation: fun() -> int
+                # or pydra decorator: @pydra.mark.annotate({"return": int})
                 else:
-                    if not isinstance(return_info, tuple):
-                        output_spec = SpecInfo(
-                            name="Output",
-                            fields=[("out", return_info)],
-                            bases=(BaseSpec,),
-                        )
-                    else:
-                        output_spec = SpecInfo(
-                            name="Output",
-                            fields=[
-                                ("out{}".format(n + 1), t)
-                                for n, t in enumerate(return_info)
-                            ],
-                            bases=(BaseSpec,),
-                        )
-        elif "return" in func.__annotations__:
-            raise NotImplementedError("Branch not implemented")
+                    output_spec = SpecInfo(
+                        name="Output", fields=[("out", return_info)], bases=(BaseSpec,)
+                    )
         self.output_spec = output_spec
 
     def _run_task(self):
@@ -219,6 +217,9 @@ class FunctionTask(TaskBase):
 class ShellCommandTask(TaskBase):
     """Wrap a shell command as a task element."""
 
+    input_spec = None
+    output_spec = None
+
     def __new__(cls, container_info=None, *args, **kwargs):
         if not container_info:
             return super().__new__(cls)
@@ -278,13 +279,28 @@ class ShellCommandTask(TaskBase):
             TODO
 
         """
-        if input_spec is None:
-            input_spec = SpecInfo(name="Inputs", fields=[], bases=(ShellSpec,))
-        self.input_spec = input_spec
-        if output_spec is None:
-            output_spec = SpecInfo(name="Output", fields=[], bases=(ShellOutSpec,))
 
-        self.output_spec = output_spec
+        # using provided spec, class attribute or setting the default SpecInfo
+        self.input_spec = (
+            input_spec
+            or self.input_spec
+            or SpecInfo(name="Inputs", fields=[], bases=(ShellSpec,))
+        )
+        self.output_spec = (
+            output_spec
+            or self.output_spec
+            or SpecInfo(name="Output", fields=[], bases=(ShellOutSpec,))
+        )
+        self.output_spec = output_from_inputfields(self.output_spec, self.input_spec)
+
+        for special_inp in ["executable", "args"]:
+            if hasattr(self, special_inp):
+                if special_inp not in kwargs:
+                    kwargs[special_inp] = getattr(self, special_inp)
+                elif kwargs[special_inp] != getattr(self, special_inp):
+                    warnings.warn(
+                        f"you are changing the executable from {getattr(self, special_inp)} to {kwargs[special_inp]}"
+                    )
 
         super().__init__(
             name=name,
@@ -397,14 +413,8 @@ class ShellCommandTask(TaskBase):
         """
         argstr = field.metadata.get("argstr", None)
         if argstr is None:
-            if "output_file_template" in field.metadata:
-                # assuming that input that has output_file_template and no arstr
-                # are not used in the command
-                return None
-            else:
-                raise Exception(
-                    f"{field.name} doesn't have argstr field in the metadata"
-                )
+            # assuming that input that has no arstr is not used in the command
+            return None
         pos = field.metadata.get("position", None)
         if pos is None:
             # position will be set at the end
@@ -480,7 +490,7 @@ class ShellCommandTask(TaskBase):
         # checking the inputs fields before returning the command line
         self.inputs.check_fields_input_spec()
         orig_inputs = attr.asdict(self.inputs)
-        modified_inputs = template_update(self.inputs)
+        modified_inputs = template_update(self.inputs, output_dir=self.output_dir)
         if modified_inputs is not None:
             self.inputs = attr.evolve(self.inputs, **modified_inputs)
         if isinstance(self, ContainerTask):
