@@ -59,9 +59,16 @@ def test_task_init_2():
     "splitter, state_splitter, state_rpn, states_ind, states_val",
     [("a", "NA.a", ["NA.a"], [{"NA.a": 0}, {"NA.a": 1}], [{"NA.a": 3}, {"NA.a": 5}])],
 )
-def test_task_init_3(splitter, state_splitter, state_rpn, states_ind, states_val):
+@pytest.mark.parametrize("input_type", ["list", "array"])
+def test_task_init_3(
+    splitter, state_splitter, state_rpn, states_ind, states_val, input_type
+):
     """ task with inputs and splitter"""
-    nn = fun_addtwo(name="NA", a=[3, 5]).split(splitter=splitter)
+    a_in = [3, 5]
+    if input_type == "array":
+        a_in = np.array(a_in)
+
+    nn = fun_addtwo(name="NA", a=a_in).split(splitter=splitter)
 
     assert np.allclose(nn.inputs.a, [3, 5])
     assert nn.state.splitter == state_splitter
@@ -101,9 +108,17 @@ def test_task_init_3(splitter, state_splitter, state_rpn, states_ind, states_val
         ),
     ],
 )
-def test_task_init_3a(splitter, state_splitter, state_rpn, states_ind, states_val):
+@pytest.mark.parametrize("input_type", ["list", "array", "mixed"])
+def test_task_init_3a(
+    splitter, state_splitter, state_rpn, states_ind, states_val, input_type
+):
     """ task with inputs and splitter"""
-    nn = fun_addvar(name="NA", a=[3, 5], b=[10, 20]).split(splitter=splitter)
+    a_in, b_in = [3, 5], [10, 20]
+    if input_type == "array":
+        a_in, b_in = np.array(a_in), np.array(b_in)
+    elif input_type == "mixed":
+        a_in = np.array(a_in)
+    nn = fun_addvar(name="NA", a=a_in, b=b_in).split(splitter=splitter)
 
     assert np.allclose(nn.inputs.a, [3, 5])
     assert np.allclose(nn.inputs.b, [10, 20])
@@ -485,7 +500,7 @@ def test_task_nostate_4(plugin, tmpdir):
     assert nn.output_dir.exists()
 
 
-def test_task_nostate_5(plugin, tmpdir):
+def test_task_nostate_5(tmpdir):
     """ task with a dictionary of files as an input"""
     file1 = tmpdir.join("file1.txt")
     with open(file1, "w") as f:
@@ -497,8 +512,7 @@ def test_task_nostate_5(plugin, tmpdir):
 
     nn = fun_file_list(name="NA", filename_list=[file1, file2])
 
-    with Submitter(plugin=plugin) as sub:
-        sub(nn)
+    nn()
 
     # checking the results
     results = nn.result()
@@ -704,9 +718,14 @@ def test_task_nostate_cachelocations_updated(plugin, tmpdir):
 
 
 @pytest.mark.flaky(reruns=2)  # when dask
-def test_task_state_1(plugin_dask_opt):
+@pytest.mark.parametrize("input_type", ["list", "array"])
+def test_task_state_1(plugin_dask_opt, input_type):
     """ task with the simplest splitter"""
-    nn = fun_addtwo(name="NA").split(splitter="a", a=[3, 5])
+    a_in = [3, 5]
+    if input_type == "array":
+        a_in = np.array(a_in)
+
+    nn = fun_addtwo(name="NA").split(splitter="a", a=a_in)
 
     assert nn.state.splitter == "NA.a"
     assert nn.state.splitter_rpn == ["NA.a"]
@@ -818,14 +837,19 @@ def test_task_state_singl_1(plugin):
         ),
     ],
 )
+@pytest.mark.parametrize("input_type", ["list", "array", "mixed"])
 def test_task_state_2(
-    plugin, splitter, state_splitter, state_rpn, expected, expected_ind
+    plugin, splitter, state_splitter, state_rpn, expected, expected_ind, input_type
 ):
     """ Tasks with two inputs and a splitter (no combiner)"""
-    nn = fun_addvar(name="NA").split(splitter=splitter, a=[3, 5], b=[10, 20])
-
-    assert nn.inputs.a == [3, 5]
-    assert nn.inputs.b == [10, 20]
+    a_in, b_in = [3, 5], [10, 20]
+    if input_type == "array":
+        a_in, b_in = np.array(a_in), np.array(b_in)
+    elif input_type == "mixed":
+        a_in = np.array(a_in)
+    nn = fun_addvar(name="NA").split(splitter=splitter, a=a_in, b=b_in)
+    assert (nn.inputs.a == np.array([3, 5])).all()
+    assert (nn.inputs.b == np.array([10, 20])).all()
     assert nn.state.splitter == state_splitter
     assert nn.state.splitter_rpn == state_rpn
     assert nn.state.splitter_final == state_splitter
@@ -879,9 +903,13 @@ def test_task_state_3(plugin):
     assert nn.output_dir == []
 
 
-def test_task_state_4(plugin):
+@pytest.mark.parametrize("input_type", ["list", "array"])
+def test_task_state_4(plugin, input_type):
     """ task with a list as an input, and a simple splitter """
-    nn = moment(name="NA", n=3, lst=[[2, 3, 4], [1, 2, 3]]).split(splitter="lst")
+    lst_in = [[2, 3, 4], [1, 2, 3]]
+    if input_type == "array":
+        lst_in = np.array(lst_in)
+    nn = moment(name="NA", n=3, lst=lst_in).split(splitter="lst")
     assert np.allclose(nn.inputs.n, 3)
     assert np.allclose(nn.inputs.lst, [[2, 3, 4], [1, 2, 3]])
     assert nn.state.splitter == "NA.lst"
@@ -889,6 +917,14 @@ def test_task_state_4(plugin):
     with Submitter(plugin=plugin) as sub:
         sub(nn)
 
+    # checking that split is done across dim 0
+    el_0 = nn.state.states_val[0]["NA.lst"]
+    if input_type == "list":
+        assert el_0 == [2, 3, 4]
+    elif input_type == "array":
+        assert isinstance(el_0, np.ndarray)
+        assert (el_0 == [2, 3, 4]).all()
+
     # checking the results
     results = nn.result()
     for i, expected in enumerate([33, 12]):
