@@ -79,10 +79,10 @@ def test_state_1(
 ):
     """ single state: testing groups, prepare_states and prepare_inputs"""
     st = State(name="NA", splitter=splitter)
-    assert st.splitter == st.right_splitter
-    assert st.splitter_rpn == st.right_splitter_rpn
-    assert st.left_splitter is None
-    assert st.left_combiner_all == []
+    assert st.splitter == st.current_splitter
+    assert st.splitter_rpn == st.current_splitter_rpn
+    assert st.prev_state_splitter is None
+    assert st.prev_state_combiner_all == []
 
     st.prepare_states(inputs)
     assert st.group_for_inputs_final == group_for_inputs
@@ -129,10 +129,10 @@ def test_state_connect_1():
     st2 = State(name="NB", other_states={"NA": (st1, "b")})
     assert st2.splitter == "_NA"
     assert st2.splitter_rpn == ["NA.a"]
-    assert st2.left_splitter == st2.splitter
-    assert st2.left_splitter_rpn == st2.splitter_rpn
-    assert st2.right_splitter is None
-    assert st2.right_splitter_rpn == []
+    assert st2.prev_state_splitter == st2.splitter
+    assert st2.prev_state_splitter_rpn == st2.splitter_rpn
+    assert st2.current_splitter is None
+    assert st2.current_splitter_rpn == []
 
     st2.prepare_states(inputs={"NA.a": [3, 5]})
     assert st2.group_for_inputs_final == {"NA.a": 0}
@@ -146,7 +146,7 @@ def test_state_connect_1():
 
 def test_state_connect_1a():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs
-        the second state has explicit splitter from the first one (Left part)
+        the second state has explicit splitter from the first one (the prev-state part)
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="_NA", other_states={"NA": (st1, "b")})
@@ -184,17 +184,17 @@ def test_state_connect_1c_exception(splitter2, other_states2):
 def test_state_connect_2():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs
          the second state has explicit splitter that contains
-         splitter from the first node and a new field (Left and Right parts)
+         splitter from the first node and a new field (the prev-state and current part)
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter=["_NA", "a"], other_states={"NA": (st1, "b")})
 
     assert st2.splitter == ["_NA", "NB.a"]
     assert st2.splitter_rpn == ["NA.a", "NB.a", "*"]
-    assert st2.left_splitter == "_NA"
-    assert st2.left_splitter_rpn == ["NA.a"]
-    assert st2.right_splitter == "NB.a"
-    assert st2.right_splitter_rpn == ["NB.a"]
+    assert st2.prev_state_splitter == "_NA"
+    assert st2.prev_state_splitter_rpn == ["NA.a"]
+    assert st2.current_splitter == "NB.a"
+    assert st2.current_splitter_rpn == ["NB.a"]
 
     st2.update_connections()
     st2.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [1, 2]})
@@ -265,16 +265,16 @@ def test_state_connect_2a():
 
 def test_state_connect_2b():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs
-        the second state has explicit splitter with a new field (Right part)
-        splitter from the first node (Left part) has to be added
+        the second state has explicit splitter with a new field (the current part)
+        splitter from the first node (the prev-state part) has to be added
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="a", other_states={"NA": (st1, "b")})
 
     assert st2.splitter == ["_NA", "NB.a"]
     assert st2.splitter_rpn == ["NA.a", "NB.a", "*"]
-    assert st2.right_splitter == "NB.a"
-    assert st2.left_splitter == "_NA"
+    assert st2.current_splitter == "NB.a"
+    assert st2.prev_state_splitter == "_NA"
 
     st2.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [1, 2]})
     assert st2.group_for_inputs_final == {"NA.a": 0, "NB.a": 1}
@@ -304,7 +304,7 @@ def test_state_connect_2b():
 def test_state_connect_3():
     """ three 'connected' states: testing groups, prepare_states and prepare_inputs
         the third state connected to two previous states;
-        splitter from the previous states (Left part) has to be added
+        splitter from the previous states (the prev-state part) has to be added
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="a")
@@ -312,11 +312,11 @@ def test_state_connect_3():
 
     assert st3.splitter == ["_NA", "_NB"]
     assert st3.splitter_rpn == ["NA.a", "NB.a", "*"]
-    assert st3.left_splitter == st3.splitter
-    assert st3.left_splitter_rpn == st3.splitter_rpn
-    assert st3.left_splitter_rpn_compact == ["_NA", "_NB", "*"]
-    assert st3.right_splitter is None
-    assert st3.right_splitter_rpn == []
+    assert st3.prev_state_splitter == st3.splitter
+    assert st3.prev_state_splitter_rpn == st3.splitter_rpn
+    assert st3.prev_state_splitter_rpn_compact == ["_NA", "_NB", "*"]
+    assert st3.current_splitter is None
+    assert st3.current_splitter_rpn == []
 
     st3.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [30, 50]})
     assert st3.group_for_inputs_final == {"NA.a": 0, "NB.a": 1}
@@ -388,7 +388,7 @@ def test_state_connect_3b():
     """ three 'connected' states: testing groups, prepare_states and prepare_inputs
         the third state connected to two previous states;
         the third state has explicit splitter that contains splitter only from the first state.
-        splitter from the second state has to be added (partial Left part)
+        splitter from the second state has to be added (partial prev-state part)
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="a")
@@ -439,11 +439,11 @@ def test_state_connect_4():
 
     assert st3.splitter == ("_NA", "_NB")
     assert st3.splitter_rpn == ["NA.a", "NB.a", "."]
-    assert st3.left_splitter == st3.splitter
-    assert st3.left_splitter_rpn == st3.splitter_rpn
-    assert st3.left_splitter_rpn_compact == ["_NA", "_NB", "."]
-    assert st3.right_splitter is None
-    assert st3.right_splitter_rpn == []
+    assert st3.prev_state_splitter == st3.splitter
+    assert st3.prev_state_splitter_rpn == st3.splitter_rpn
+    assert st3.prev_state_splitter_rpn_compact == ["_NA", "_NB", "."]
+    assert st3.current_splitter is None
+    assert st3.current_splitter_rpn == []
 
     st3.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [30, 50]})
     assert st3.group_for_inputs_final == {"NA.a": 0, "NB.a": 0}
@@ -595,11 +595,11 @@ def test_state_connect_innerspl_1():
 
     assert st2.splitter == ["_NA", "NB.b"]
     assert st2.splitter_rpn == ["NA.a", "NB.b", "*"]
-    assert st2.left_splitter == "_NA"
-    assert st2.left_splitter_rpn == ["NA.a"]
-    assert st2.left_splitter_rpn_compact == ["_NA"]
-    assert st2.right_splitter == "NB.b"
-    assert st2.right_splitter_rpn == ["NB.b"]
+    assert st2.prev_state_splitter == "_NA"
+    assert st2.prev_state_splitter_rpn == ["NA.a"]
+    assert st2.prev_state_splitter_rpn_compact == ["_NA"]
+    assert st2.current_splitter == "NB.b"
+    assert st2.current_splitter_rpn == ["NB.b"]
 
     st2.prepare_states(
         inputs={"NA.a": [3, 5], "NB.b": [[1, 10, 100], [2, 20, 200]]},
@@ -640,18 +640,18 @@ def test_state_connect_innerspl_1():
 def test_state_connect_innerspl_1a():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs,
         the second state has an inner splitter,
-        splitter from the first state (Left part) has to be added
+        splitter from the first state (the prev-state part) has to be added
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="b", other_states={"NA": (st1, "b")})
 
     assert st2.splitter == ["_NA", "NB.b"]
     assert st2.splitter_rpn == ["NA.a", "NB.b", "*"]
-    assert st2.left_splitter == "_NA"
-    assert st2.left_splitter_rpn == ["NA.a"]
-    assert st2.left_splitter_rpn_compact == ["_NA"]
-    assert st2.right_splitter == "NB.b"
-    assert st2.right_splitter_rpn == ["NB.b"]
+    assert st2.prev_state_splitter == "_NA"
+    assert st2.prev_state_splitter_rpn == ["NA.a"]
+    assert st2.prev_state_splitter_rpn_compact == ["_NA"]
+    assert st2.current_splitter == "NB.b"
+    assert st2.current_splitter_rpn == ["NB.b"]
 
     assert st2.other_states["NA"][1] == "b"
 
@@ -691,7 +691,7 @@ def test_state_connect_innerspl_1a():
 
 
 def test_state_connect_innerspl_1b():
-    """incorrect splitter - Right & Left parts in scalar splitter"""
+    """incorrect splitter - the current & prev-state parts in scalar splitter"""
     with pytest.raises(PydraStateError):
         st1 = State(name="NA", splitter="a")
         st2 = State(name="NB", splitter=("_NA", "b"), other_states={"NA": (st1, "b")})
@@ -700,18 +700,18 @@ def test_state_connect_innerspl_1b():
 def test_state_connect_innerspl_2():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs,
         the second state has one inner splitter and one 'normal' splitter
-        only Right part of the splitter provided (Left has to be added)
+        only the current part of the splitter provided (the prev-state has to be added)
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter=["c", "b"], other_states={"NA": (st1, "b")})
 
     assert st2.splitter == ["_NA", ["NB.c", "NB.b"]]
     assert st2.splitter_rpn == ["NA.a", "NB.c", "NB.b", "*", "*"]
-    assert st2.left_splitter == "_NA"
-    assert st2.left_splitter_rpn == ["NA.a"]
-    assert st2.left_splitter_rpn_compact == ["_NA"]
-    assert st2.right_splitter == ["NB.c", "NB.b"]
-    assert st2.right_splitter_rpn == ["NB.c", "NB.b", "*"]
+    assert st2.prev_state_splitter == "_NA"
+    assert st2.prev_state_splitter_rpn == ["NA.a"]
+    assert st2.prev_state_splitter_rpn_compact == ["_NA"]
+    assert st2.current_splitter == ["NB.c", "NB.b"]
+    assert st2.current_splitter_rpn == ["NB.c", "NB.b", "*"]
 
     st2.prepare_states(
         inputs={"NA.a": [3, 5], "NB.b": [[1, 10, 100], [2, 20, 200]], "NB.c": [13, 17]},
@@ -770,7 +770,7 @@ def test_state_connect_innerspl_2():
 def test_state_connect_innerspl_2a():
     """ two 'connected' states: testing groups, prepare_states and prepare_inputs,
         the second state has one inner splitter and one 'normal' splitter
-        only Right part of the splitter provided (different order!),
+        only the current part of the splitter provided (different order!),
 
     """
     st1 = State(name="NA", splitter="a")
@@ -837,7 +837,7 @@ def test_state_connect_innerspl_2a():
 def test_state_connect_innerspl_3():
     """ three serially 'connected' states: testing groups, prepare_states and prepare_inputs,
         the second state has one inner splitter and one 'normal' splitter
-        Left parts have to be added
+        the prev-state parts of the splitter have to be added
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter=["c", "b"], other_states={"NA": (st1, "b")})
@@ -845,11 +845,11 @@ def test_state_connect_innerspl_3():
 
     assert st3.splitter == ["_NB", "NC.d"]
     assert st3.splitter_rpn == ["NA.a", "NB.c", "NB.b", "*", "*", "NC.d", "*"]
-    assert st3.left_splitter == "_NB"
-    assert st3.left_splitter_rpn == ["NA.a", "NB.c", "NB.b", "*", "*"]
-    assert st3.left_splitter_rpn_compact == ["_NB"]
-    assert st3.right_splitter == "NC.d"
-    assert st3.right_splitter_rpn == ["NC.d"]
+    assert st3.prev_state_splitter == "_NB"
+    assert st3.prev_state_splitter_rpn == ["NA.a", "NB.c", "NB.b", "*", "*"]
+    assert st3.prev_state_splitter_rpn_compact == ["_NB"]
+    assert st3.current_splitter == "NC.d"
+    assert st3.current_splitter_rpn == ["NC.d"]
 
     st3.prepare_states(
         inputs={
@@ -976,7 +976,7 @@ def test_state_connect_innerspl_3():
 
 def test_state_connect_innerspl_4():
     """ three'connected' states: testing groups, prepare_states and prepare_inputs,
-        the third one connected to two previous, only Right part of splitter provided
+        the third one connected to two previous, only the current part of splitter provided
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter=["b", "c"])
@@ -1066,8 +1066,8 @@ def test_state_combine_1():
     st = State(name="NA", splitter="a", combiner="a")
     assert st.splitter == "NA.a"
     assert st.splitter_rpn == ["NA.a"]
-    assert st.right_combiner == st.right_combiner_all == st.combiner == ["NA.a"]
-    assert st.left_combiner == st.left_combiner_all == []
+    assert st.current_combiner == st.current_combiner_all == st.combiner == ["NA.a"]
+    assert st.prev_state_combiner == st.prev_state_combiner_all == []
     assert st.splitter_final == None
     assert st.splitter_rpn_final == []
 
@@ -1090,8 +1090,8 @@ def test_state_connect_combine_1():
     assert st1.splitter_rpn == ["NA.a", "NA.b", "*"]
     assert st1.splitter_rpn_final == ["NA.b"]
     assert st1.splitter_final == "NA.b"
-    assert st1.combiner == st1.right_combiner == st1.right_combiner_all == ["NA.a"]
-    assert st1.left_combiner_all == st1.left_combiner == []
+    assert st1.combiner == st1.current_combiner == st1.current_combiner_all == ["NA.a"]
+    assert st1.prev_state_combiner_all == st1.prev_state_combiner == []
     assert st2.splitter == "_NA"
     assert st2.splitter_rpn == ["NA.b"]
 
@@ -1202,8 +1202,8 @@ def test_state_connect_combine_3():
     assert st2.splitter == ["_NA", "NB.d"]
     assert st2.splitter_rpn == ["NA.b", "NB.d", "*"]
     assert st2.splitter_rpn_final == ["NA.b"]
-    assert st2.left_combiner_all == st2.left_combiner == []
-    assert st2.right_combiner_all == st2.right_combiner == st2.combiner == ["NB.d"]
+    assert st2.prev_state_combiner_all == st2.prev_state_combiner == []
+    assert st2.current_combiner_all == st2.current_combiner == st2.combiner == ["NB.d"]
 
     st2.prepare_states(
         inputs={"NA.a": [3, 5], "NA.b": [10, 20], "NB.c": [90, 150], "NB.d": [0, 1]}
@@ -1254,7 +1254,7 @@ def test_state_connect_combine_3():
 
 def test_state_connect_innerspl_combine_1():
     """one previous node and one inner splitter (and inner splitter combiner);
-    only Right part provided - Left had to be added"""
+    only current part provided - the prev-state part had to be added"""
     st1 = State(name="NA", splitter="a")
     st2 = State(
         name="NB", splitter=["c", "b"], combiner=["b"], other_states={"NA": (st1, "b")}
@@ -1264,8 +1264,8 @@ def test_state_connect_innerspl_combine_1():
     assert st2.splitter_rpn == ["NA.a", "NB.c", "NB.b", "*", "*"]
     assert st2.splitter_final == ["NA.a", "NB.c"]
     assert st2.splitter_rpn_final == ["NA.a", "NB.c", "*"]
-    assert st2.left_combiner_all == st2.left_combiner == []
-    assert st2.right_combiner_all == st2.right_combiner == st2.combiner == ["NB.b"]
+    assert st2.prev_state_combiner_all == st2.prev_state_combiner == []
+    assert st2.current_combiner_all == st2.current_combiner == st2.combiner == ["NB.b"]
     # TODO: i think at the end I should merge [0] and [1], because there are no inner splitters anymore
     # TODO: didn't include it in my code...
     # assert st2.groups_stack_final == [[0, 1]]
@@ -1333,7 +1333,8 @@ def test_state_connect_innerspl_combine_1():
 def test_state_connect_innerspl_combine_2():
     """ two 'connected' state, the second has inner and normal splitter,
         and 'normal' combiner
-        only Right part splitter provided, Left has to be added
+        only the current part of the splitter provided,
+        the prev-state part has to be added
         """
     st1 = State(name="NA", splitter="a")
     st2 = State(
@@ -1405,18 +1406,23 @@ def test_state_connect_innerspl_combine_2():
     ]
 
 
-def test_state_connect_combine_left_1():
+def test_state_connect_combine_prevst_1():
     """ two 'connected' states,
         the first one has the simplest splitter,
         the second has combiner from the first state
-        (i.e. from the Left part of the splitter),
+        (i.e. from the prev-state part of the splitter),
     """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", other_states={"NA": (st1, "b")}, combiner="NA.a")
     assert st2.splitter == "_NA"
     assert st2.splitter_rpn == ["NA.a"]
-    assert st2.combiner == st2.left_combiner == st2.left_combiner_all == ["NA.a"]
-    assert st2.right_combiner == st2.right_combiner_all == []
+    assert (
+        st2.combiner
+        == st2.prev_state_combiner
+        == st2.prev_state_combiner_all
+        == ["NA.a"]
+    )
+    assert st2.current_combiner == st2.current_combiner_all == []
     assert st2.splitter_rpn_final == []
 
     st2.prepare_states(inputs={"NA.a": [3, 5]})
@@ -1430,19 +1436,19 @@ def test_state_connect_combine_left_1():
     assert st2.inputs_ind == [{"NB.b": 0}, {"NB.b": 1}]
 
 
-def test_state_connect_combine_left_2():
+def test_state_connect_combine_prevst_2():
     """ two 'connected' states,
         the first one has outer splitter,
         the second has combiner from the first state
-        (i.e. from the Left part of the splitter),
+        (i.e. from the prev-state part of the splitter),
     """
     st1 = State(name="NA", splitter=["a", "b"])
     st2 = State(name="NB", other_states={"NA": (st1, "b")}, combiner="NA.a")
     assert st2.splitter == "_NA"
     assert st2.splitter_rpn == ["NA.a", "NA.b", "*"]
     assert st2.combiner == ["NA.a"]
-    assert st2.left_combiner_all == st2.left_combiner == ["NA.a"]
-    assert st2.right_combiner_all == st2.right_combiner == []
+    assert st2.prev_state_combiner_all == st2.prev_state_combiner == ["NA.a"]
+    assert st2.current_combiner_all == st2.current_combiner == []
     assert st2.splitter_rpn_final == ["NA.b"]
 
     st2.prepare_states(inputs={"NA.a": [3, 5], "NA.b": [10, 20]})
@@ -1466,11 +1472,11 @@ def test_state_connect_combine_left_2():
     assert st2.inputs_ind == [{"NB.b": 0}, {"NB.b": 1}, {"NB.b": 2}, {"NB.b": 3}]
 
 
-def test_state_connect_combine_left_3():
+def test_state_connect_combine_prevst_3():
     """ three serially 'connected' states,
         the first one has outer splitter,
         the third one has combiner from the first state
-        (i.e. from the Left part of the splitter),
+        (i.e. from the prev-state part of the splitter),
     """
     st1 = State(name="NA", splitter=["a", "b"])
     st2 = State(name="NB", other_states={"NA": (st1, "b")})
@@ -1502,11 +1508,11 @@ def test_state_connect_combine_left_3():
     assert st3.inputs_ind == [{"NC.c": 0}, {"NC.c": 1}, {"NC.c": 2}, {"NC.c": 3}]
 
 
-def test_state_connect_combine_left_4():
+def test_state_connect_combine_prevst_4():
     """ three 'connected' states: testing groups, prepare_states and prepare_inputs,
          the first two states have the simplest splitters,
-         the third state has only Left splitter,
-         the third state has also combiner from the Left part
+         the third state has only the prev-state part of splitter,
+         the third state has also combiner from the prev-state part
       """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="a")
@@ -1519,8 +1525,13 @@ def test_state_connect_combine_left_4():
     assert st3.splitter == ["_NA", "_NB"]
     assert st3.splitter_rpn == ["NA.a", "NB.a", "*"]
     assert st3.splitter_rpn_final == ["NB.a"]
-    assert st3.left_combiner_all == st3.left_combiner == st3.combiner == ["NA.a"]
-    assert st3.right_combiner_all == st3.right_combiner == []
+    assert (
+        st3.prev_state_combiner_all
+        == st3.prev_state_combiner
+        == st3.combiner
+        == ["NA.a"]
+    )
+    assert st3.current_combiner_all == st3.current_combiner == []
 
     st3.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [600, 700]})
     assert st3.group_for_inputs_final == {"NB.a": 0}
@@ -1549,11 +1560,11 @@ def test_state_connect_combine_left_4():
     ]
 
 
-def test_state_connect_combine_left_5():
+def test_state_connect_combine_prevst_5():
     """ three 'connected' states: testing groups, prepare_states and prepare_inputs,
         the first two states have the simplest splitters,
-        the third state has scalar splitter in the Left part,
-        the third state has also combiner from the Left part
+        the third state has scalar splitter in the prev-state part,
+        the third state has also combiner from the prev-state part
      """
     st1 = State(name="NA", splitter="a")
     st2 = State(name="NB", splitter="a")
@@ -1567,9 +1578,9 @@ def test_state_connect_combine_left_5():
     assert st3.splitter_rpn == ["NA.a", "NB.a", "."]
     st3.set_input_groups()
     assert st3.splitter_rpn_final == []
-    assert set(st3.left_combiner_all) == {"NA.a", "NB.a"}
-    assert st3.left_combiner == st3.combiner == ["NA.a"]
-    assert st3.right_combiner_all == st3.right_combiner == []
+    assert set(st3.prev_state_combiner_all) == {"NA.a", "NB.a"}
+    assert st3.prev_state_combiner == st3.combiner == ["NA.a"]
+    assert st3.current_combiner_all == st3.current_combiner == []
 
     st3.prepare_states(inputs={"NA.a": [3, 5], "NB.a": [600, 700]})
     assert st3.group_for_inputs_final == {}
@@ -1583,11 +1594,11 @@ def test_state_connect_combine_left_5():
     assert st3.inputs_ind == [{"NC.a": 0, "NC.b": 0}, {"NC.a": 1, "NC.b": 1}]
 
 
-def test_state_connect_combine_left_6():
+def test_state_connect_combine_prevst_6():
     """ two 'connected' states,
-        the first one has outer splitter, the second has an additional Right splitter,
+        the first one has outer splitter, the second has an additional current splitter,
         the second also has combiner from the first state
-        (i.e. from the Left part of the splitter),
+        (i.e. from the prev-state part of the splitter),
     """
     st1 = State(name="NA", splitter=["a", "b"])
     st2 = State(
@@ -1596,8 +1607,8 @@ def test_state_connect_combine_left_6():
     assert st2.splitter == ["_NA", "NB.c"]
     assert st2.splitter_rpn == ["NA.a", "NA.b", "*", "NB.c", "*"]
     assert st2.combiner == ["NA.a"]
-    assert st2.left_combiner_all == st2.left_combiner == ["NA.a"]
-    assert st2.right_combiner_all == st2.right_combiner == []
+    assert st2.prev_state_combiner_all == st2.prev_state_combiner == ["NA.a"]
+    assert st2.current_combiner_all == st2.current_combiner == []
     assert st2.splitter_rpn_final == ["NA.b", "NB.c", "*"]
 
     st2.prepare_states(inputs={"NA.a": [3, 5], "NA.b": [10, 20], "NB.c": [0, 1]})
@@ -1645,7 +1656,7 @@ def test_state_connect_combine_left_6():
 
 
 @pytest.mark.parametrize(
-    "splitter, other_states, expected_splitter, expected_left, expected_right",
+    "splitter, other_states, expected_splitter, expected_prevst, expected_current",
     [
         (None, {"NA": (State(name="NA", splitter="a"), "b")}, "_NA", "_NA", None),
         (
@@ -1695,13 +1706,13 @@ def test_state_connect_combine_left_6():
     ],
 )
 def test_connect_splitters(
-    splitter, other_states, expected_splitter, expected_left, expected_right
+    splitter, other_states, expected_splitter, expected_prevst, expected_current
 ):
     st = State(name="CN", splitter=splitter, other_states=other_states)
     st.set_input_groups()
     assert st.splitter == expected_splitter
-    assert st.left_splitter == expected_left
-    assert st.right_splitter == expected_right
+    assert st.prev_state_splitter == expected_prevst
+    assert st.current_splitter == expected_current
 
 
 @pytest.mark.parametrize(
@@ -1721,7 +1732,7 @@ def test_connect_splitters(
 def test_connect_splitters_exception_1(splitter, other_states):
     with pytest.raises(PydraStateError) as excinfo:
         st = State(name="CN", splitter=splitter, other_states=other_states)
-    assert "Left and Right splitters are mixed" in str(excinfo.value)
+    assert "prev-state and current splitters are mixed" in str(excinfo.value)
 
 
 def test_connect_splitters_exception_2():
