@@ -1,11 +1,13 @@
 import typing as ty
 import os, sys
+import attr
 import pytest
 
 from ... import mark
 from ..task import AuditFlag, ShellCommandTask, DockerTask, SingularityTask
 from ...utils.messenger import FileMessenger, PrintMessenger, collect_messages
 from .utils import gen_basic_wf, use_validator
+from ..specs import MultiInputObj, MultiOutputObj, SpecInfo, FunctionSpec, BaseSpec
 
 no_win = pytest.mark.skipif(
     sys.platform.startswith("win"),
@@ -167,10 +169,8 @@ def test_annotated_input_func_2a(use_validator):
         return a
 
     funky = testfunc()
-    # the error is raised when run (should be improved?)
-    funky.inputs.a = 3.5
     with pytest.raises(TypeError):
-        funky()
+        funky.inputs.a = 3.5
 
 
 def test_annotated_input_func_3(use_validator):
@@ -334,6 +334,54 @@ def test_annotated_input_func_7a_excep(use_validator):
         funky = testfunc(a=[3.5, 2.1]).split("a")
 
 
+def test_annotated_input_func_8():
+    """ the function with annotated input as MultiInputObj
+        a single value is provided and should be converted to a list
+    """
+
+    @mark.task
+    def testfunc(a: MultiInputObj):
+        return len(a)
+
+    funky = testfunc(a=3.5)
+    assert getattr(funky.inputs, "a") == [3.5]
+    res = funky()
+    assert res.output.out == 1
+
+
+def test_annotated_input_func_8a():
+    """ the function with annotated input as MultiInputObj
+        a 1-el list is provided so shouldn't be changed
+    """
+
+    @mark.task
+    def testfunc(a: MultiInputObj):
+        return len(a)
+
+    funky = testfunc(a=[3.5])
+    assert getattr(funky.inputs, "a") == [3.5]
+    res = funky()
+    assert res.output.out == 1
+
+
+def test_annotated_input_func_8b():
+    """ the function with annotated input as MultiInputObj
+        a single value is provided after initial. the task
+        (input should still be converted to a list)
+    """
+
+    @mark.task
+    def testfunc(a: MultiInputObj):
+        return len(a)
+
+    funky = testfunc()
+    # setting a after init
+    funky.inputs.a = 3.5
+    assert getattr(funky.inputs, "a") == [3.5]
+    res = funky()
+    assert res.output.out == 1
+
+
 def test_annotated_func_multreturn_exception(use_validator):
     """function has two elements in the return statement,
         but three element provided in the spec - should raise an error
@@ -499,6 +547,385 @@ def test_notannotated_func_multreturn():
     assert result.output.out == (20.2, 13.8)
 
 
+def test_input_spec_func_1(use_validator):
+    """ the function w/o annotated, but input_spec is used """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[("a", attr.ib(type=float, metadata={"help_string": "input a"}))],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=3.5, input_spec=my_input_spec)
+    assert getattr(funky.inputs, "a") == 3.5
+
+
+def test_input_spec_func_1a_except(use_validator):
+    """ the function w/o annotated, but input_spec is used
+    a TypeError is raised (float is provided instead of int)
+    """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[("a", attr.ib(type=int, metadata={"help_string": "input a"}))],
+        bases=(FunctionSpec,),
+    )
+    with pytest.raises(TypeError):
+        funky = testfunc(a=3.5, input_spec=my_input_spec)
+
+
+def test_input_spec_func_1b_except(use_validator):
+    """ the function w/o annotated, but input_spec is used
+     metadata checks raise an error
+     """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "a",
+                attr.ib(type=float, metadata={"position": 1, "help_string": "input a"}),
+            )
+        ],
+        bases=(FunctionSpec,),
+    )
+    with pytest.raises(AttributeError, match="only these keys are supported"):
+        funky = testfunc(a=3.5, input_spec=my_input_spec)
+
+
+def test_input_spec_func_1d_except(use_validator):
+    """ the function w/o annotated, but input_spec is used
+    input_spec doesn't contain 'a' input, an error is raised
+    """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(name="Input", fields=[], bases=(FunctionSpec,))
+    funky = testfunc(a=3.5, input_spec=my_input_spec)
+    with pytest.raises(TypeError, match="missing 1 required positional argument"):
+        funky()
+
+
+def test_input_spec_func_2(use_validator):
+    """ the function with annotation, and the task has input_spec,
+    input_spec changes the type of the input (so error is not raised)
+    """
+
+    @mark.task
+    def testfunc(a: int):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[("a", attr.ib(type=float, metadata={"help_string": "input a"}))],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=3.5, input_spec=my_input_spec)
+    assert getattr(funky.inputs, "a") == 3.5
+
+
+def test_input_spec_func_2a(use_validator):
+    """ the function with annotation, and the task has input_spec,
+    input_spec changes the type of the input (so error is not raised)
+    using the shorter syntax
+    """
+
+    @mark.task
+    def testfunc(a: int):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[("a", float, {"help_string": "input a"})],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=3.5, input_spec=my_input_spec)
+    assert getattr(funky.inputs, "a") == 3.5
+
+
+def test_input_spec_func_3(use_validator):
+    """ the function w/o annotated, but input_spec is used
+    additional keys (allowed_values) are used in metadata
+    """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "a",
+                attr.ib(
+                    type=int,
+                    metadata={"help_string": "input a", "allowed_values": [0, 1, 2]},
+                ),
+            )
+        ],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=2, input_spec=my_input_spec)
+    assert getattr(funky.inputs, "a") == 2
+
+
+def test_input_spec_func_3a_except(use_validator):
+    """ the function w/o annotated, but input_spec is used
+    allowed_values is used in metadata and the ValueError is raised
+    """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "a",
+                attr.ib(
+                    type=int,
+                    metadata={"help_string": "input a", "allowed_values": [0, 1, 2]},
+                ),
+            )
+        ],
+        bases=(FunctionSpec,),
+    )
+
+    with pytest.raises(ValueError, match="value of a has to be"):
+        funky = testfunc(a=3, input_spec=my_input_spec)
+
+
+def test_input_spec_func_4(use_validator):
+    """ the function with a default value for b
+    but b is set as mandatory in the input_spec, so error is raised if not provided
+    """
+
+    @mark.task
+    def testfunc(a, b=1):
+        return a + b
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "a",
+                attr.ib(
+                    type=int, metadata={"help_string": "input a", "mandatory": True}
+                ),
+            ),
+            (
+                "b",
+                attr.ib(
+                    type=int, metadata={"help_string": "input b", "mandatory": True}
+                ),
+            ),
+        ],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=2, input_spec=my_input_spec)
+    with pytest.raises(Exception, match="b is mandatory"):
+        funky()
+
+
+def test_input_spec_func_4a(use_validator):
+    """ the function with a default value for b and metadata in the input_spec
+    has a different default value, so value from the function is overwritten
+    """
+
+    @mark.task
+    def testfunc(a, b=1):
+        return a + b
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            (
+                "a",
+                attr.ib(
+                    type=int, metadata={"help_string": "input a", "mandatory": True}
+                ),
+            ),
+            ("b", attr.ib(type=int, default=10, metadata={"help_string": "input b"})),
+        ],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=2, input_spec=my_input_spec)
+    res = funky()
+    assert res.output.out == 12
+
+
+def test_input_spec_func_5():
+    """ the FunctionTask with input_spec, a input has MultiInputObj type
+        a single value is provided and should be converted to a list
+    """
+
+    @mark.task
+    def testfunc(a):
+        return len(a)
+
+    my_input_spec = SpecInfo(
+        name="Input",
+        fields=[
+            ("a", attr.ib(type=MultiInputObj, metadata={"help_string": "input a"}))
+        ],
+        bases=(FunctionSpec,),
+    )
+
+    funky = testfunc(a=3.5, input_spec=my_input_spec)
+    assert getattr(funky.inputs, "a") == [3.5]
+    res = funky()
+    assert res.output.out == 1
+
+
+def test_output_spec_func_1(use_validator):
+    """ the function w/o annotated, but output_spec is used """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[("out1", attr.ib(type=float, metadata={"help_string": "output"}))],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, output_spec=my_output_spec)
+    res = funky()
+    assert res.output.out1 == 3.5
+
+
+def test_output_spec_func_1a_except(use_validator):
+    """ the function w/o annotated, but output_spec is used
+        float returned instead of int - TypeError
+    """
+
+    @mark.task
+    def testfunc(a):
+        return a
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[("out1", attr.ib(type=int, metadata={"help_string": "output"}))],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, output_spec=my_output_spec)
+    with pytest.raises(TypeError):
+        res = funky()
+
+
+def test_output_spec_func_2(use_validator):
+    """ the function w/o annotated, but output_spec is used
+    output_spec changes the type of the output (so error is not raised)
+    """
+
+    @mark.task
+    def testfunc(a) -> int:
+        return a
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[("out1", attr.ib(type=float, metadata={"help_string": "output"}))],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, output_spec=my_output_spec)
+    res = funky()
+    assert res.output.out1 == 3.5
+
+
+def test_output_spec_func_2a(use_validator):
+    """ the function w/o annotated, but output_spec is used
+    output_spec changes the type of the output (so error is not raised)
+    using a shorter syntax
+    """
+
+    @mark.task
+    def testfunc(a) -> int:
+        return a
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[("out1", float, {"help_string": "output"})],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, output_spec=my_output_spec)
+    res = funky()
+    assert res.output.out1 == 3.5
+
+
+def test_output_spec_func_3(use_validator):
+    """ the function w/o annotated, but output_spec is used
+    MultiOutputObj is used, output is a 2-el list, so converter doesn't do anything
+    """
+
+    @mark.task
+    def testfunc(a, b):
+        return [a, b]
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[
+            (
+                "out_list",
+                attr.ib(type=MultiOutputObj, metadata={"help_string": "output"}),
+            )
+        ],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, b=1, output_spec=my_output_spec)
+    res = funky()
+    assert res.output.out_list == [3.5, 1]
+
+
+def test_output_spec_func_4(use_validator):
+    """ the function w/o annotated, but output_spec is used
+    MultiOutputObj is used, output is a 1el list, so converter return the element
+    """
+
+    @mark.task
+    def testfunc(a):
+        return [a]
+
+    my_output_spec = SpecInfo(
+        name="Output",
+        fields=[
+            (
+                "out_1el",
+                attr.ib(type=MultiOutputObj, metadata={"help_string": "output"}),
+            )
+        ],
+        bases=(BaseSpec,),
+    )
+
+    funky = testfunc(a=3.5, output_spec=my_output_spec)
+    res = funky()
+    assert res.output.out_1el == 3.5
+
+
 def test_exception_func():
     @mark.task
     def raise_exception(c, d):
@@ -663,7 +1090,7 @@ def test_functask_callable(tmpdir):
     assert foo2.plugin == "cf"
 
 
-def test_taskhooks(tmpdir, capsys):
+def test_taskhooks_1(tmpdir, capsys):
     foo = funaddtwo(name="foo", a=1, cache_dir=tmpdir)
     assert foo.hooks
     # ensure all hooks are defined
@@ -715,3 +1142,78 @@ def test_taskhooks(tmpdir, capsys):
     for attr in ("pre_run", "post_run", "pre_run_task", "post_run_task"):
         hook = getattr(foo.hooks, attr)
         assert hook() is None
+
+
+def test_taskhooks_2(tmpdir, capsys):
+    """checking order of the hooks; using task's attributes"""
+    foo = funaddtwo(name="foo", a=1, cache_dir=tmpdir)
+
+    def myhook_prerun(task, *args):
+        print(f"i. prerun hook was called from {task.name}")
+
+    def myhook_prerun_task(task, *args):
+        print(f"ii. prerun task hook was called {task.name}")
+
+    def myhook_postrun_task(task, *args):
+        print(f"iii. postrun task hook was called {task.name}")
+
+    def myhook_postrun(task, *args):
+        print(f"iv. postrun hook was called {task.name}")
+
+    foo.hooks.pre_run = myhook_prerun
+    foo.hooks.post_run = myhook_postrun
+    foo.hooks.pre_run_task = myhook_prerun_task
+    foo.hooks.post_run_task = myhook_postrun_task
+    foo()
+
+    captured = capsys.readouterr()
+    hook_messages = captured.out.strip().split("\n")
+    # checking the order of the hooks
+    assert "i. prerun hook" in hook_messages[0]
+    assert "ii. prerun task hook" in hook_messages[1]
+    assert "iii. postrun task hook" in hook_messages[2]
+    assert "iv. postrun hook" in hook_messages[3]
+
+
+def test_taskhooks_3(tmpdir, capsys):
+    """checking results in the post run hooks"""
+    foo = funaddtwo(name="foo", a=1, cache_dir=tmpdir)
+
+    def myhook_postrun_task(task, result, *args):
+        print(f"postrun task hook, the result is {result.output.out}")
+
+    def myhook_postrun(task, result, *args):
+        print(f"postrun hook, the result is {result.output.out}")
+
+    foo.hooks.post_run = myhook_postrun
+    foo.hooks.post_run_task = myhook_postrun_task
+    foo()
+
+    captured = capsys.readouterr()
+    hook_messages = captured.out.strip().split("\n")
+    # checking that the postrun hooks have access to results
+    assert "postrun task hook, the result is 3" in hook_messages[0]
+    assert "postrun hook, the result is 3" in hook_messages[1]
+
+
+def test_taskhooks_4(tmpdir, capsys):
+    """task raises an error: postrun task should be called, postrun shouldn't be called"""
+    foo = funaddtwo(name="foo", a="one", cache_dir=tmpdir)
+
+    def myhook_postrun_task(task, result, *args):
+        print(f"postrun task hook was called, result object is {result}")
+
+    def myhook_postrun(task, result, *args):
+        print(f"postrun hook should not be called")
+
+    foo.hooks.post_run = myhook_postrun
+    foo.hooks.post_run_task = myhook_postrun_task
+
+    with pytest.raises(Exception):
+        foo()
+
+    captured = capsys.readouterr()
+    hook_messages = captured.out.strip().split("\n")
+    # only post run task hook should be called
+    assert len(hook_messages) == 1
+    assert "postrun task hook was called" in hook_messages[0]
